@@ -1,6 +1,6 @@
 /*** yuck-scmver.c -- snarf versions off project cwds
  *
- * Copyright (C) 2013-2014 Sebastian Freundt
+ * Copyright (C) 2013-2016 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -48,7 +48,7 @@
 #include <fcntl.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
-#include <yuck-scmver.h>
+#include "yuck-scmver.h"
 
 #if !defined LIKELY
 # define LIKELY(_x)	__builtin_expect((_x), 1)
@@ -74,6 +74,14 @@
 
 #define DEBUG(args...)
 
+/* globals */
+const char *const yscm_strs[] = {
+	[YUCK_SCM_TARBALL] = "tarball",
+	[YUCK_SCM_GIT] = "git",
+	[YUCK_SCM_BZR] = "bzr",
+	[YUCK_SCM_HG] = "hg",
+};
+
 
 static __attribute__((format(printf, 1, 2))) void
 error(const char *fmt, ...)
@@ -137,9 +145,11 @@ xdirname(char *restrict fn, const char *fp)
 /* find next dir in FN from FP backwards */
 	if (fp == NULL) {
 		fp = fn + strlen(fn);
+	} else if (fp <= fn) {
+		return NULL;
 	}
 
-	while (--fp >= fn && *fp != '/');
+	for (--fp; fp >= fn && *fp != '/'; fp--);
 	while (fp >= fn && *--fp == '/');
 	if (fp >= fn) {
 		/* replace / by \nul and return pointer */
@@ -276,6 +286,7 @@ run(int *fd, ...)
 	     i < countof(cmdline) &&
 		     (cmdline[i] = va_arg(vap, char*)) != NULL; i++);
 	va_end(vap);
+	assert(*cmdline);
 
 	if (pipe(intfd) < 0) {
 		error("pipe setup to/from %s failed", cmdline[0U]);
@@ -486,6 +497,11 @@ rd_version(struct yuck_version_s *restrict v, const char *buf, size_t bsz)
 	case '\0':
 		return 0;
 	case '.':
+		if (v->scm <= YUCK_SCM_TARBALL) {
+			/* huh? */
+			return -1;
+		}
+		/*@fallthrough@*/
 	case '-':
 		/* the show is going on, like it must */
 		bp = eod + 1U;
@@ -501,9 +517,12 @@ rd_version(struct yuck_version_s *restrict v, const char *buf, size_t bsz)
 		v->scm = YUCK_SCM_HG;
 		break;
 	case 'b':
-		/* bzr repo */
-		v->scm = YUCK_SCM_BZR;
-		break;
+		if (v->scm <= YUCK_SCM_TARBALL) {
+			v->scm = YUCK_SCM_BZR;
+			break;
+		}
+		/* else probably git or hg hash starting with b */
+		/*@fallthrough@*/
 	default:
 		/* could have been set already then */
 		if (v->scm > YUCK_SCM_TARBALL) {
@@ -583,7 +602,7 @@ git_version(struct yuck_version_s v[static 1U])
 	int rc = 0;
 
 	if ((chld = run(fd, "git", "describe",
-			"--match=v[0-9]*",
+			"--tags", "--match=v[0-9]*",
 			"--abbrev=8", "--dirty", NULL)) < 0) {
 		return -1;
 	}
@@ -667,6 +686,7 @@ hg_version(struct yuck_version_s v[static 1U])
 
 		if ((nrd = read(*fd, buf, sizeof(buf))) <= 0) {
 			/* no version then aye */
+			rc = -1;
 			break;
 		}
 		buf[nrd - 1U/* for \n*/] = '\0';
@@ -787,7 +807,6 @@ bzr_version(struct yuck_version_s v[static 1U])
 			break;
 		}
 		/* read over all the whitespace to find the tag's revno */
-		while (*++bp == ' ');
 		with (unsigned int rno = strtoul(bp, NULL, 10)) {
 			v->dist = v->rvsn - rno;
 		}
@@ -862,7 +881,9 @@ yuck_version_read(struct yuck_version_s *restrict ref, const char *fn)
 	/* initialise result structure */
 	memset(ref, 0, sizeof(*ref));
 
-	if ((fd = open(fn, O_RDONLY)) < 0) {
+	if (fn[0U] == '-' && fn[1U] == '\0') {
+		fd = STDIN_FILENO;
+	} else if ((fd = open(fn, O_RDONLY)) < 0) {
 		return -1;
 	}
 	/* otherwise read and parse the string */
@@ -878,8 +899,11 @@ yuck_version_read(struct yuck_version_s *restrict ref, const char *fn)
 			/* just go with the first line */
 			*bp = '\0';
 			nrd = bp - buf;
-		} else {
+		} else if ((size_t)nrd < sizeof(buf)) {
 			/* finalise with \nul */
+			buf[nrd] = '\0';
+		} else {
+			/* finalise with \nul, cutting off the last byte */
 			buf[--nrd] = '\0';
 		}
 		/* otherwise just read him */
@@ -909,7 +933,9 @@ yuck_version_write(const char *fn, const struct yuck_version_s *ref)
 	int rc = 0;
 	int fd;
 
-	if ((fd = open(fn, O_RDWR | O_CREAT | O_TRUNC, 0666)) < 0) {
+	if (fn[0U] == '-' && fn[1U] == '\0') {
+		fd = STDOUT_FILENO;
+	} else if ((fd = open(fn, O_RDWR | O_CREAT | O_TRUNC, 0666)) < 0) {
 		return -1;
 	}
 	if (yuck_version_write_fd(fd, ref) < 0) {
@@ -932,13 +958,6 @@ yuck_version_cmp(yuck_version_t v1, yuck_version_t v2)
 
 
 #if defined BOOTSTRAP
-static const char *yscm_strs[] = {
-	[YUCK_SCM_TARBALL] = "tarball",
-	[YUCK_SCM_GIT] = "git",
-	[YUCK_SCM_BZR] = "bzr",
-	[YUCK_SCM_HG] = "hg",
-};
-
 int
 main(int argc, char *argv[])
 {
@@ -946,18 +965,14 @@ main(int argc, char *argv[])
 	static struct yuck_version_s v[1U];
 	int rc = 0;
 
-	switch ((rc = yuck_version(v, argv[1U]))) {
-	default:
-		/* one more chance there is */
-		if (argc <= 2) {
-			/* no reference file given */
-			break;
-		} else if ((rc = yuck_version_read(v, argv[2U])) < 0) {
-			/* ok, can't help it then */
-			break;
-		}
-		/* yay, fallthrough to success */
-	case 0:
+	/* prefer reference file */
+	if (argc > 2 && (rc = yuck_version_read(v, argv[2U])) == 0) {
+		/* just use this one */
+		;
+	} else {
+		rc = yuck_version(v, argv[1U]);
+	}
+	if (rc == 0) {
 		fputs("define(YUCK_SCMVER_VERSION, ", stdout);
 		fputs(v->vtag, stdout);
 		if (v->scm > YUCK_SCM_TARBALL && v->dist) {
@@ -971,10 +986,46 @@ main(int argc, char *argv[])
 			fputs(".dirty", stdout);
 		}
 		fputs(")\n", stdout);
-		break;
 	}
 	return -rc;
 }
 #endif	/* BOOTSTRAP */
 
+
+#if defined CONFIGURE
+int
+main(int argc, char *argv[])
+{
+/* usage would be yuck-scmver [REFERENCE] */
+	static struct yuck_version_s v[1U];
+	int rc = 0;
+
+	if (argc > 1) {
+		rc = yuck_version_read(v, argv[1U]);
+#if defined VERSION_FILE
+	} else if ((rc = yuck_version_read(v, VERSION_FILE)) == 0) {
+		;
+#endif	/* VERSION_FILE */
+	} else {
+		rc = yuck_version(v, NULL);
+	}
+	/* print if successful */
+	if (rc == 0) {
+		fputs(v->vtag, stdout);
+		if (v->scm > YUCK_SCM_TARBALL && v->dist) {
+			fputc('.', stdout);
+			fputs(yscm_strs[v->scm], stdout);
+			fprintf(stdout, "%u.%0*x",
+				v->dist,
+				(int)(v->rvsn & 0x07U), v->rvsn >> 4U);
+		}
+		if (v->dirty) {
+			fputs(".dirty", stdout);
+		}
+		fputc('\n', stdout);
+	}
+	return -rc;
+}
+#endif	/* CONFIGURE */
+
 /* yuck-scmver.c ends here */
