@@ -1,6 +1,6 @@
 /*** yuck.c -- generate umbrella commands
  *
- * Copyright (C) 2013-2014 Sebastian Freundt
+ * Copyright (C) 2013-2016 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -79,6 +79,9 @@
 #if !defined countof
 # define countof(x)	(sizeof(x) / sizeof(*x))
 #endif	/* !countof */
+#if !defined strlenof
+# define strlenof(x)	(sizeof(x) - 1ULL)
+#endif	/* !strlenof */
 
 #define _paste(x, y)	x ## y
 #define paste(x, y)	_paste(x, y)
@@ -117,14 +120,7 @@ struct opt_s {
 	unsigned int marg:1U;
 };
 
-#if !defined BOOTSTRAP && defined WITH_SCMVER
-static const char *yscm_strs[] = {
-	[YUCK_SCM_TARBALL] = "tarball",
-	[YUCK_SCM_GIT] = "git",
-	[YUCK_SCM_BZR] = "bzr",
-	[YUCK_SCM_HG] = "hg",
-};
-#elif !defined BOOTSTRAP
+#if !defined BOOTSTRAP && !defined WITH_SCMVER
 /* just forward declare this type so function signatures will work */
 struct yuck_version_s;
 #endif	/* WITH_SCMVER */
@@ -170,6 +166,9 @@ max_zu(size_t x, size_t y)
 static size_t
 xstrncpy(char *restrict dst, const char *src, size_t ssz)
 {
+	if (UNLIKELY(dst == NULL)) {
+		return 0U;
+	}
 	memcpy(dst, src, ssz);
 	dst[ssz] = '\0';
 	return ssz;
@@ -362,35 +361,42 @@ typedef struct {
 	/* the actual buffer (resizable) */
 	char *s;
 	/* current size */
+	size_t n;
+	/* current alloc size */
 	size_t z;
 }  bbuf_t;
 
-static char*
-bbuf_cpy(bbuf_t b[static 1U], const char *str, size_t ssz)
+static __attribute__((nonnull(1, 2))) char*
+bbuf_cat(bbuf_t *restrict b, const char *str, size_t ssz)
 {
-	size_t nu = max_zu(yfls(ssz + 1U) + 1U, 6U);
-	size_t ol = b->z ? max_zu(yfls(b->z) + 1U, 6U) : 0U;
+	if (UNLIKELY(b->n + ssz + 1U/*\nul*/ > b->z)) {
+		const size_t nu = max_zu(yfls(b->n + ssz + 1U) + 1U, 6U);
+		char *tmp;
 
-	if (UNLIKELY(nu > ol)) {
-		b->s = realloc(b->s, (1U << nu) * sizeof(*b->s));
+		tmp = realloc(b->s, (b->z = (1ULL << nu) * sizeof(*b->s)));
+		if (UNLIKELY(tmp == NULL)) {
+			goto free;
+		}
+		/* all's good then? */
+		b->s = tmp;
 	}
-	xstrncpy(b->s, str, ssz);
-	b->z += ssz;
+	xstrncpy(b->s + b->n, str, ssz);
+	b->n += ssz;
 	return b->s;
+free:
+	free(b->s);
+	b->s = NULL;
+	b->n = 0U;
+	b->z = 0U;
+	return NULL;
 }
 
-static char*
-bbuf_cat(bbuf_t b[static 1U], const char *str, size_t ssz)
+static __attribute__((nonnull(1, 2))) char*
+bbuf_cpy(bbuf_t *restrict b, const char *str, size_t ssz)
 {
-	size_t nu = max_zu(yfls(b->z + ssz + 1U) + 1U, 6U);
-	size_t ol = b->z ? max_zu(yfls(b->z) + 1U, 6U) : 0U;
-
-	if (UNLIKELY(nu > ol)) {
-		b->s = realloc(b->s, (1U << nu) * sizeof(*b->s));
-	}
-	xstrncpy(b->s + b->z, str, ssz);
-	b->z += ssz;
-	return b->s;
+/* reduce to bbuf_cat() with zero offset */
+	b->n = 0U;
+	return bbuf_cat(b, str, ssz);
 }
 
 
@@ -427,7 +433,7 @@ usagep(const char *line, size_t llen)
 		/* it's a setopt */
 		return 0;
 	} else if (!STREQLITP(line, "usage:")) {
-		if (only_whitespace_p(line, llen) && !desc->z) {
+		if (only_whitespace_p(line, llen) && !desc->n) {
 			return 1;
 		} else if (!isspace(*line) && !cur_usg_yldd_p) {
 			/* append to description */
@@ -435,7 +441,7 @@ usagep(const char *line, size_t llen)
 			return 1;
 		}
 	yield:
-#define RESET	cur_usg.cmd = cur_usg.parg = cur_usg.desc = NULL, desc->z = 0U
+#define RESET	cur_usg.cmd = cur_usg.parg = cur_usg.desc = NULL, desc->n = 0U
 
 		if (!cur_usg_yldd_p) {
 			yield_usg(&cur_usg);
@@ -498,8 +504,9 @@ overread:
 	if (cur_usg.cmd && !strncasecmp(cur_usg.cmd, cp, sp - cp)) {
 		/* nothing new and fresh */
 		;
-	} else if ((*cp != '<' || (cp++, *--sp == '>')) &&
-		   !strncasecmp(cp, "command", sp - cp)) {
+	} else if (!strncasecmp(cp, "command", sp - cp) ||
+		   (cp[0] == '<' && sp[-1] == '>' &&
+		    !strncasecmp(cp + 1U, "command", sp - cp - 2))) {
 		/* special command COMMAND or <command> */
 		cur_usg.cmd = NULL;
 	} else if (*cp >= 'a' && *cp <= 'z' && umb_yldd_p) {
@@ -573,7 +580,7 @@ yield:
 			sp++;
 		}
 		if (!isspace(*sp)) {
-			/* dont know -x.SOMETHING? */
+			/* don't know -x.SOMETHING? */
 			return 0;
 		}
 		/* start over with the new option */
@@ -618,7 +625,7 @@ yield:
 		/* --option */
 		;
 	} else {
-		/* dont know what this is */
+		/* don't know what this is */
 		return 0;
 	}
 
@@ -662,8 +669,8 @@ yield:
 	;
 	/* space eater */
 	for (; sp < ep && isspace(*sp); sp++);
-	/* dont free but reset the old guy */
-	desc->z = 0U;
+	/* don't free but reset the old guy */
+	desc->n = 0U;
 desc:
 	with (size_t sz = llen - (sp - line)) {
 		if (LIKELY(sz > 0U)) {
@@ -684,11 +691,11 @@ interp(const char *line, size_t llen)
 	}
 
 	DEBUG("INTERP CALLED with %s", line);
-	if (only_ws_p && desc->z) {
+	if (only_ws_p && desc->n) {
 	yield:
 		yield_inter(desc);
 		/* reset */
-		desc->z = 0U;
+		desc->n = 0U;
 	} else if (!only_ws_p) {
 		if (STREQLITP(line, "setopt")) {
 			/* not an inter */
@@ -739,6 +746,9 @@ static struct {
 static void
 __identify(char *restrict idn)
 {
+	if (UNLIKELY(idn == NULL)) {
+		return;
+	}
 	for (char *restrict ip = idn; *ip; ip++) {
 		switch (*ip) {
 		case '0' ... '9':
@@ -802,14 +812,18 @@ make_opt_ident(const struct opt_s *arg)
 	if (arg->lopt != NULL) {
 		bbuf_cpy(i, arg->lopt, strlen(arg->lopt));
 	} else if (arg->sopt) {
-		bbuf_cpy(i, "dash.", 5U);
-		i->s[4U] = arg->sopt;
+		if (bbuf_cpy(i, "dash.", 5U) != NULL) {
+			i->s[4U] = arg->sopt;
+		}
 	} else {
 		static unsigned int cnt;
-		bbuf_cpy(i, "idnXXXX", 7U);
-		snprintf(i->s + 3U, 5U, "%u", cnt++);
+		if (bbuf_cpy(i, "idnXXXX", 7U) != NULL) {
+			snprintf(i->s + 3U, 5U, "%u", cnt++);
+		}
+	}
+	if (LIKELY(i->s != NULL)) {
+		__identify(i->s);
 	}
-	__identify(i->s);
 	return i->s;
 }
 
@@ -818,8 +832,9 @@ make_ident(const char *str)
 {
 	static bbuf_t buf[1U];
 
-	bbuf_cpy(buf, str, strlen(str));
-	__identify(buf->s);
+	if (LIKELY(bbuf_cpy(buf, str, strlen(str)) != NULL)) {
+		__identify(buf->s);
+	}
 	return buf->s;
 }
 
@@ -922,9 +937,9 @@ yield_opt(const struct opt_s *arg)
 static void
 yield_inter(const bbuf_t x[static 1U])
 {
-	if (x->z) {
-		if (x->s[x->z - 1U] == '\n') {
-			x->s[x->z - 1U] = '\0';
+	if (x->n) {
+		if (x->s[x->n - 1U] == '\n') {
+			x->s[x->n - 1U] = '\0';
 		}
 		massage_desc(x->s);
 		fprintf(outf, "yuck_add_inter([%s])\n", x->s);
@@ -1120,9 +1135,54 @@ get_myself(char *restrict buf, size_t bsz)
 	ssize_t off;
 	char *mp;
 
-	if ((off = readlink("/proc/self/exe", buf, bsz)) < 0) {
+#if defined __linux__
+	static const char myself[] = "/proc/self/exe";
+
+	if (UNLIKELY((off = readlink(myself, buf, bsz)) < 0)) {
+		/* shame */
 		return -1;
 	}
+#elif defined __NetBSD__
+	static const char myself[] = "/proc/curproc/exe";
+
+	if (UNLIKELY((off = readlink(myself, buf, bsz)) < 0)) {
+		/* nawww */
+		return -1;
+	}
+#elif defined __DragonFly__
+	static const char myself[] = "/proc/curproc/file";
+
+	if (UNLIKELY((off = readlink(myself, buf, bsz)) < 0)) {
+		/* blimey */
+		return -1;
+	}
+#elif defined __FreeBSD__
+	int mib[] = {CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1};
+
+	/* make sure that \0 terminator fits */
+	buf[--bsz] = '\0';
+	if (UNLIKELY(sysctl(mib, countof(mib), buf, &bsz, NULL, 0) < 0)) {
+		return -1;
+	}
+	/* we can be grateful they gave us the path, counting is our job */
+	off = strlen(buf);
+#elif defined __sun || defined sun
+
+	snprintf(buf, bsz, "/proc/%d/path/a.out", getpid());
+	if (UNLIKELY((off = readlink(buf, buf, bsz)) < 0)) {
+		return -1;
+	}
+#elif defined __APPLE__ && defined __MACH__
+	uint32_t z = --bsz;
+	if (_NSGetExecutablePath(buf, &z) != 0) {
+		return -1;
+	}
+	/* good, do the counting */
+	off = strlen(buf);
+#else
+	return -1;
+#endif
+
 	/* go back to the dir bit */
 	for (mp = buf + off - 1U; mp > buf && *mp != '/'; mp--);
 	/* should be bin/, go up one level */
@@ -1185,8 +1245,8 @@ find_aux(char *restrict buf, size_t bsz, const char *aux)
 	}
 #if defined YUCK_TEMPLATE_PATH
 	path = YUCK_TEMPLATE_PATH;
-	plen = sizeof(YUCK_TEMPLATE_PATH);
-	if (plen-- > 0U && aux_in_path_p(aux, path, plen)) {
+	plen = strlenof(YUCK_TEMPLATE_PATH);
+	if (aux_in_path_p(aux, path, plen)) {
 		goto bang;
 	}
 #endif	/* YUCK_TEMPLATE_PATH */
@@ -1227,7 +1287,7 @@ unmassage_fd(int tgtfd, int srcfd)
 
 
 static char *m4_cmdline[16U] = {
-	"m4",
+	YUCK_M4,
 };
 static size_t cmdln_idx;
 
@@ -1276,8 +1336,10 @@ static __attribute__((noinline)) int
 run_m4(const char *outfn, ...)
 {
 	pid_t m4p;
-	/* to snarf off traffic from the child */
+	/* we need a bidirectional pipe (for the unmassaging) */
 	int intfd[2];
+	int outfd = STDOUT_FILENO;
+	int rc;
 
 	if (pipe(intfd) < 0) {
 		error("pipe setup to/from m4 failed");
@@ -1287,51 +1349,7 @@ run_m4(const char *outfn, ...)
 		return -1;
 	}
 
-	switch ((m4p = vfork())) {
-	case -1:
-		/* i am an error */
-		error("vfork for m4 failed");
-		return -1;
-
-	default:;
-		/* i am the parent */
-		int rc;
-		int st;
-
-		if (outfn != NULL) {
-			/* --output given */
-			const int outfl = O_RDWR | O_CREAT | O_TRUNC;
-			int outfd;
-
-			if ((outfd = open(outfn, outfl, 0666)) < 0) {
-				/* bollocks */
-				error("cannot open outfile `%s'", outfn);
-				goto bollocks;
-			}
-
-			/* really redir now */
-			dup2(outfd, STDOUT_FILENO);
-			close(outfd);
-		}
-
-		close(intfd[1]);
-		unmassage_fd(STDOUT_FILENO, intfd[0]);
-
-		rc = 2;
-		while (waitpid(m4p, &st, 0) != m4p);
-		if (WIFEXITED(st)) {
-			rc = WEXITSTATUS(st);
-		}
-		/* clean up the rest of the pipe */
-		close(intfd[0]);
-		return rc;
-
-	case 0:;
-		/* i am the child */
-		break;
-	}
-
-	/* child code here */
+	/* command-line prepping */
 	with (va_list vap) {
 		va_start(vap, outfn);
 		for (size_t i = cmdln_idx;
@@ -1340,13 +1358,55 @@ run_m4(const char *outfn, ...)
 		va_end(vap);
 	}
 
-	dup2(intfd[1], STDOUT_FILENO);
-	close(intfd[0]);
+	switch ((m4p = vfork())) {
+	case -1:
+		/* i am an error */
+		error("vfork for m4 failed");
+		return -1;
 
-	execvp(m4_cmdline[0U], m4_cmdline);
-	error("execvp(m4) failed");
-bollocks:
-	_exit(EXIT_FAILURE);
+	default:
+		break;
+
+	case 0:
+		/* redirect stdout -> intfd[1] */
+		dup2(intfd[1], STDOUT_FILENO);
+		close(intfd[1]);
+		close(intfd[0]);
+		/* i am the child */
+		execvp(m4_cmdline[0U], m4_cmdline);
+		error("execvp(m4) failed");
+		_exit(EXIT_FAILURE);
+	}
+
+	/* i am the parent */
+	close(intfd[1]);
+
+	/* prep redirection */
+	if (outfn != NULL) {
+		/* --output given */
+		const int outfl = O_RDWR | O_CREAT | O_TRUNC;
+
+		if ((outfd = open(outfn, outfl, 0666)) < 0) {
+			/* bollocks */
+			error("cannot open outfile `%s'", outfn);
+			return -1;
+		}
+	}
+
+	/* reroute m4's output */
+	unmassage_fd(outfd, intfd[0]);
+
+	rc = 2;
+	with (int st) {
+		while (waitpid(m4p, &st, 0) != m4p);
+		if (WIFEXITED(st)) {
+			rc = WEXITSTATUS(st);
+		}
+	}
+	if (outfn != NULL) {
+		close(outfd);
+	}
+	return rc;
 }
 
 
@@ -1875,13 +1935,22 @@ flag -n|--use-reference requires -r|--reference parameter");
 		/* must populate v then */
 		*v = *ref;
 	} else if (reffn && yuck_version_cmp(v, ref)) {
+		/* version stamps differ */
 		if (argi->verbose_flag) {
 			errno = 0, error("scm version differs from reference");
 		}
-		/* version stamps differ */
-		yuck_version_write(argi->reference_arg, v);
-		/* reserve exit code 3 for `updated reference file' */
-		rc = 3;
+		/* try to update the reference file then */
+		if (yuck_version_write(argi->reference_arg, v) < 0) {
+			if (argi->verbose_flag) {
+				error("cannot write reference file");
+			}
+			/* degrade to using the reference file */
+			*v = *ref;
+			rc = 0;
+		} else {
+			/* reserve exit code 3 for `updated reference file' */
+			rc = 3;
+		}
 	} else if (reffn && !argi->force_flag) {
 		/* make sure the output file exists */
 		const char *const outfn = argi->output_arg;
@@ -1914,7 +1983,7 @@ flag -n|--use-reference requires -r|--reference parameter");
 			/* and we're finished with the intermediary */
 			fclose(outf);
 			/* macro massage, vtmpfn is the template file */
-			rc = run_m4(outfn, scmvfn, tmplfn, infn, NULL);
+			rc += run_m4(outfn, scmvfn, tmplfn, infn, NULL);
 
 			rm_intermediary(scmvfn, argi->keep_flag);
 		}
@@ -1939,6 +2008,24 @@ flag -n|--use-reference requires -r|--reference parameter");
 #endif	/* WITH_SCMVER */
 }
 
+static int
+cmd_config(const struct yuck_cmd_config_s argi[static 1U])
+{
+	const char *const fn = argi->output_arg;
+	FILE *of = stdout;
+
+	if (fn != NULL && (of = fopen(fn, "w")) == NULL) {
+		error("cannot open file `%s'", fn);
+		return -1;
+	}
+
+	if (argi->m4_flag) {
+		fputs(YUCK_M4, of);
+		fputc('\n', of);
+	}
+	return fclose(of);
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -1977,6 +2064,11 @@ See --help to obtain a list of available commands.\n", stderr);
 			rc = 1;
 		}
 		break;
+	case YUCK_CMD_CONFIG:
+		if ((rc = cmd_config((const void*)argi)) < 0) {
+			rc = 1;
+		}
+		break;
 	}
 
 out:
