@@ -1,6 +1,6 @@
 /*** date-core.c -- our universe of dates
  *
- * Copyright (C) 2011-2014 Sebastian Freundt
+ * Copyright (C) 2011-2016 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -47,6 +47,8 @@
 #include <sys/time.h>
 #include <time.h>
 #include <assert.h>
+#include <errno.h>
+#include <limits.h>
 #include "date-core.h"
 #include "date-core-private.h"
 #include "strops.h"
@@ -242,8 +244,10 @@ dt_get_md(struct dt_d_s that)
 		return (struct __md_s){.m = that.ymcw.m, .d = d};
 	}
 	case DT_YWD:
-		/* should have come throught the GETTERS aspect */
+		/* should have come through the GETTERS aspect */
 		return __ywd_get_md(that.ywd);
+	case DT_YD:
+		return __yd_get_md(that.yd);
 	}
 }
 
@@ -452,6 +456,8 @@ dt_conv_to_daisy(struct dt_d_s that)
 		return __ldn_to_daisy(that.ldn);
 	case DT_JDN:
 		return __jdn_to_daisy(that.jdn);
+	case DT_YD:
+		return __yd_to_daisy(that.yd);
 	case DT_DUNK:
 	default:
 		break;
@@ -479,6 +485,8 @@ dt_conv_to_ymd(struct dt_d_s that)
 		return __bizda_to_ymd(that.bizda);
 	case DT_YWD:
 		return __ywd_to_ymd(that.ywd);
+	case DT_YD:
+		return __yd_to_ymd(that.yd);
 	case DT_DUNK:
 	default:
 		break;
@@ -506,6 +514,8 @@ dt_conv_to_ymcw(struct dt_d_s that)
 		return __bizda_to_ymcw(that.bizda, __get_bizda_param(that));
 	case DT_YWD:
 		return __ywd_to_ymcw(that.ywd);
+	case DT_YD:
+		return __yd_to_ymcw(that.yd);
 	case DT_DUNK:
 	default:
 		break;
@@ -528,6 +538,8 @@ dt_conv_to_bizda(struct dt_d_s that)
 		break;
 	case DT_DAISY:
 		break;
+	case DT_YD:
+		break;
 	case DT_DUNK:
 	default:
 		break;
@@ -556,6 +568,8 @@ dt_conv_to_ywd(struct dt_d_s this)
 		return __daisy_to_ywd(this.daisy);
 	case DT_BIZDA:
 		return __bizda_to_ywd(this.bizda, __get_bizda_param(this));
+	case DT_YD:
+		return __yd_to_ywd(this.yd);
 	case DT_DUNK:
 	default:
 		break;
@@ -631,15 +645,15 @@ dt_conv_to_yd(struct dt_d_s this)
 
 DEFVAR const char ymd_dflt[] = "%F";
 DEFVAR const char ymcw_dflt[] = "%Y-%m-%c-%w";
-DEFVAR const char ywd_dflt[] = "%rY-W%V-%u";
-DEFVAR const char yd_dflt[] = "%Y-%d";
+DEFVAR const char ywd_dflt[] = "%G-W%V-%u";
+DEFVAR const char yd_dflt[] = "%Y-%D";
 DEFVAR const char daisy_dflt[] = "%d";
 DEFVAR const char bizsi_dflt[] = "%db";
 DEFVAR const char bizda_dflt[] = "%Y-%m-%db";
 
 DEFVAR const char ymddur_dflt[] = "%Y-%0m-%0d";
 DEFVAR const char ymcwdur_dflt[] = "%Y-%0m-%0w-%0d";
-DEFVAR const char ywddur_dflt[] = "%rY-W%0w-%0d";
+DEFVAR const char ywddur_dflt[] = "%G-W%0w-%d";
 DEFVAR const char yddur_dflt[] = "%Y-%0d";
 DEFVAR const char daisydur_dflt[] = "%d";
 DEFVAR const char bizsidur_dflt[] = "%db";
@@ -704,7 +718,7 @@ __trans_dfmt(const char **fmt)
 	return DT_DUNK;
 }
 
-DEFUN dt_dtyp_t
+DEFUN dt_durtyp_t
 __trans_ddurfmt(const char **fmt)
 {
 	if (UNLIKELY(*fmt == NULL)) {
@@ -714,37 +728,44 @@ __trans_ddurfmt(const char **fmt)
 		/* don't worry about it */
 		;
 	} else {
-		const dt_dtyp_t tmp = __trans_dfmt_special(*fmt);
+		unsigned int tmp = __trans_dfmt_special(*fmt);
 
 		switch (tmp) {
 		default:
 			break;
 		case DT_YMD:
 			*fmt = ymddur_dflt;
+			tmp = DT_DURYMD;
 			break;
 		case DT_YMCW:
 			*fmt = ymcwdur_dflt;
+			tmp = DT_DURYMCW;
 			break;
 		case DT_YWD:
 			*fmt = ywddur_dflt;
+			tmp = DT_DURYWD;
 			break;
 		case DT_YD:
 			*fmt = yddur_dflt;
+			tmp = DT_DURYD;
 			break;
 		case DT_BIZDA:
 			*fmt = bizdadur_dflt;
+			tmp = DT_DURBIZDA;
 			break;
 		case DT_DAISY:
 			*fmt = daisydur_dflt;
+			tmp = DT_DURD;
 			break;
 		case DT_BIZSI:
 			*fmt = bizsidur_dflt;
+			tmp = DT_DURBD;
 			break;
 		}
 
-		return tmp;
+		return (dt_durtyp_t)tmp;
 	}
-	return DT_DUNK;
+	return DT_DURUNK;
 }
 
 /* strpf glue */
@@ -758,7 +779,6 @@ __guess_dtyp(struct strpd_s d)
 		res.typ = DT_YMD;
 		res.ymd.y = d.y;
 		if (LIKELY(!d.flags.d_dcnt_p)) {
-			res.ymd.m = d.m;
 #if defined WITH_FAST_ARITH
 			res.ymd.d = d.d;
 #else  /* !WITH_FAST_ARITH */
@@ -768,13 +788,23 @@ __guess_dtyp(struct strpd_s d)
 				res.ymd.d = md;
 				res.fix = 1U;
 			}
-		} else {
-			/* convert dcnt to m + d */
-			struct __md_s r = __yday_get_md(d.y, d.d);
-			res.ymd.m = r.m;
-			res.ymd.d = r.d;
-		}
 #endif	/* !WITH_FAST_ARITH */
+			/* month is always pertained */
+			res.ymd.m = d.m;
+		} else {
+			/* produce yd dates */
+			res.typ = DT_YD;
+			res.yd.y = d.y;
+#if defined WITH_FAST_ARITH
+			res.yd.d = d.d;
+#else  /* !WITH_FAST_ARITH */
+			with (int maxd = __get_ydays(d.y)) {
+				if (UNLIKELY((res.yd.d = d.d) > maxd)) {
+					res.yd.d = maxd;
+				}
+			}
+#endif	/* WITH_FAST_ARITH */
+		}
 	} else if (d.y > 0 && d.m <= 0 && !d.flags.bizda) {
 		res.typ = DT_YWD;
 		res.ywd = __make_ywd_c(d.y, d.c, (dt_dow_t)d.w, d.flags.wk_cnt);
@@ -786,7 +816,7 @@ __guess_dtyp(struct strpd_s d)
 #if defined WITH_FAST_ARITH
 		res.ymcw.c = d.c;
 #else  /* !WITH_FAST_ARITH */
-		if ((res.ymcw.c = d.c) >= 5) {
+		if (UNLIKELY((res.ymcw.c = d.c) >= 5)) {
 			/* the user meant the LAST wday actually */
 			res.ymcw.c = __get_mcnt(d.y, d.m, (dt_dow_t)d.w);
 		}
@@ -933,6 +963,9 @@ dt_strfd(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 		case DT_YWD:
 			fmt = ywd_dflt;
 			break;
+		case DT_YD:
+			fmt = yd_dflt;
+			break;
 		case DT_DAISY:
 			/* subject to change */
 			fmt = ymd_dflt;
@@ -959,6 +992,11 @@ dt_strfd(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 		d.c = that.ymcw.c;
 		d.w = that.ymcw.w;
 		break;
+	case DT_YD:
+		d.y = that.yd.y;
+		d.d = that.yd.d;
+		d.flags.d_dcnt_p = 1U;
+		break;
 	case DT_JDN:
 		that.typ = DT_DAISY;
 		that.daisy = __jdn_to_daisy(that.jdn);
@@ -1016,80 +1054,68 @@ dt_strfd(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 	return bp - buf;
 }
 
-DEFUN struct dt_d_s
+DEFUN struct dt_ddur_s
 dt_strpddur(const char *str, char **ep)
 {
 /* at the moment we allow only one format */
-	struct dt_d_s res = dt_d_initialiser();
-	struct strpd_s d = strpd_initialiser();
+	struct dt_ddur_s res = dt_make_ddur(DT_DURUNK, 0);
 	const char *sp = str;
-	int tmp;
+	long int tmp;
 
 	if (str == NULL) {
 		goto out;
 	}
-	/* read just one component */
-	tmp = strtol(sp, (char**)&sp, 10);
+	/* read off co-class indicator */
+	if (*sp == '/') {
+		res.cocl = 1U;
+		sp++;
+	}
+	/* read just one component, use rudi's errno trick */
+	errno = 0;
+	if ((tmp = strtol(str, (char**)&sp, 10)) == 0 && str == sp) {
+		/* didn't work aye? */
+		goto out;
+	} else if (tmp > INT_MAX || errno) {
+		errno = ERANGE;
+		goto out;
+	}
+
 	switch (*sp++) {
 	case '\0':
 		/* must have been day then */
-		d.d = tmp;
+		res.durtyp = DT_DURD;
 		sp--;
 		break;
 	case 'd':
 	case 'D':
-		d.d = tmp;
+		res.durtyp = DT_DURD;
 		break;
 	case 'y':
 	case 'Y':
-		d.y = tmp;
+		res.durtyp = DT_DURYR;
 		break;
 	case 'm':
 	case 'M':
-		d.m = tmp;
+		res.durtyp = DT_DURMO;
 		break;
 	case 'w':
 	case 'W':
-		d.w = tmp;
+		res.durtyp = DT_DURWK;
 		break;
 	case 'b':
 	case 'B':
-		d.b = tmp;
+		res.durtyp = DT_DURBD;
 		break;
 	case 'q':
 	case 'Q':
-		d.q = tmp;
+		res.durtyp = DT_DURQU;
 		break;
 	default:
 		sp = str;
 		goto out;
 	}
-	/* assess */
-	if (d.b && (d.y || d.m >= 1)) {
-		res.typ = DT_BIZDA;
-		res.bizda.y = d.y;
-		res.bizda.m = d.q * 3 + d.m;
-		res.bizda.bd = d.b + d.w * DUWW_BDAYS_P_WEEK;
-	} else if (LIKELY(d.y || d.m >= 1)) {
-		res.typ = DT_YMD;
-		res.ymd.y = d.y;
-		res.ymd.m = d.q * 3 + d.m;
-		res.ymd.d = d.d + d.w * GREG_DAYS_P_WEEK;
-	} else if (d.d) {
-		res.typ = DT_DAISY;
-		res.daisydur = d.w * GREG_DAYS_P_WEEK + d.d;
-	} else if (d.b) {
-		res.typ = DT_BIZSI;
-		res.bizsidur = d.w * DUWW_BDAYS_P_WEEK + d.b;
-	} else {
-		/* we leave out YMCW diffs simply because YMD diffs
-		 * cover them better
-		 * anything that doesn't fit shall be mapped to MD durs
-		 * using the definitions of MONTHS_P_YEAR and DAYS_P_WEEK */
-		res.typ = DT_MD;
-		res.md.d = d.w * GREG_DAYS_P_WEEK + d.d;
-		res.md.m = d.y * GREG_MONTHS_P_YEAR + d.m;
-	}
+	/* no further checks on TMP */
+	res.dv = tmp;
 out:
 	if (ep != NULL) {
 		*ep = (char*)sp;
@@ -1098,18 +1124,18 @@ out:
 }
 
 DEFUN size_t
-dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
+dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_ddur_s that)
 {
 	struct strpd_s d = strpd_initialiser();
 	const char *fp;
 	char *bp;
 
-	if (UNLIKELY(buf == NULL || bsz == 0 || !that.dur)) {
+	if (UNLIKELY(buf == NULL || bsz == 0)) {
 		return 0;
 	}
 
-	switch (that.typ) {
-	case DT_YMD:
+	switch (that.durtyp) {
+	case DT_DURYMD:
 		d.y = that.ymd.y;
 		d.m = that.ymd.m;
 		d.d = that.ymd.d;
@@ -1117,7 +1143,7 @@ dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 			fmt = ymddur_dflt;
 		}
 		break;
-	case DT_YMCW:
+	case DT_DURYMCW:
 		d.y = that.ymcw.y;
 		d.m = that.ymcw.m;
 		d.c = that.ymcw.c;
@@ -1126,7 +1152,7 @@ dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 			fmt = ymcwdur_dflt;
 		}
 		break;
-	case DT_YWD:
+	case DT_DURYWD:
 		d.y = that.ywd.y;
 		d.c = that.ywd.c;
 		d.d = that.ywd.w;
@@ -1134,41 +1160,17 @@ dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 			fmt = ywddur_dflt;
 		}
 		break;
-	case DT_YD:
+	case DT_DURYD:
 		d.y = that.yd.y;
 		d.d = that.yd.d;
 		if (fmt == NULL) {
 			fmt = yddur_dflt;
 		}
 		break;
-	case DT_DAISY:
-		if (that.daisydur >= 0) {
-			d.d = that.daisydur;
-		} else {
-			d.d = -that.daisydur;
-			/* make sure the neg bit doesn't bite us */
-			that.neg = 1;
-		}
-		if (fmt == NULL) {
-			/* subject to change */
-			fmt = daisydur_dflt;
-		}
-		break;
-	case DT_BIZSI:
-		if (that.bizsidur >= 0) {
-			d.d = that.bizsidur;
-		} else {
-			d.d = -that.bizsidur;
-			/* make sure the neg bit doesn't bite us */
-			that.neg = 1;
-		}
-		if (fmt == NULL) {
-			/* subject to change */
-			fmt = bizsidur_dflt;
-		}
-		break;
-	case DT_BIZDA: {
-		dt_bizda_param_t bparam = __get_bizda_param(that);
+	case DT_DURBIZDA:;
+		dt_bizda_param_t bparam;
+
+		bparam.bs = that.param;
 		d.y = that.bizda.y;
 		d.m = that.bizda.m;
 		d.b = that.bizda.bd;
@@ -1182,13 +1184,41 @@ dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 			fmt = bizdadur_dflt;
 		}
 		break;
-	}
-	case DT_MD:
-		d.y = __uidiv(that.md.m, GREG_MONTHS_P_YEAR);
-		d.m = __uimod(that.md.m, GREG_MONTHS_P_YEAR);
-		d.w = __uidiv(that.md.d, GREG_DAYS_P_WEEK);
-		d.d = __uimod(that.md.d, GREG_DAYS_P_WEEK);
+
+	case DT_DURD:
+	case DT_DURBD:
+	case DT_DURWK:
+	case DT_DURMO:
+	case DT_DURQU:
+	case DT_DURYR:
+		if (that.dv >= 0) {
+			/* make sure the neg bit doesn't bite us */
+			that.dv = -that.dv;
+			that.neg = 1U;
+		}
+
+		switch (that.durtyp) {
+		case DT_DURD:
+			d.d = that.dv;
+			break;
+		case DT_DURBD:
+			d.b = that.dv;
+			break;
+		case DT_DURWK:
+			d.d = that.dv * GREG_DAYS_P_WEEK;
+			break;
+		case DT_DURMO:
+			d.m = that.dv;
+			break;
+		case DT_DURQU:
+			d.m = that.dv * 3U;
+			break;
+		case DT_DURYR:
+			d.y = that.dv;
+			break;
+		}
 		break;
+
 	default:
 	case DT_DUNK:
 		bp = buf;
@@ -1246,7 +1276,9 @@ dt_date(dt_dtyp_t outtyp)
 
 	switch ((res.typ = outtyp)) {
 	case DT_YMD:
-	case DT_YMCW: {
+	case DT_YMCW:
+	case DT_YWD:
+	case DT_YD: {
 		struct tm tm;
 		ffff_gmtime(&tm, t);
 		switch (res.typ) {
@@ -1274,18 +1306,32 @@ dt_date(dt_dtyp_t outtyp)
 			res.ymcw.w = tm.tm_wday;
 			break;
 		}
+		case DT_YD:
+			res.yd.y = tm.tm_year;
+			res.yd.d = tm.tm_yday;
+			break;
+		case DT_YWD:
+			/* use ordinary conversion to ywd */
+			res.typ = DT_YMD;
+			res.ymd.y = tm.tm_year;
+			res.ymd.m = tm.tm_mon;
+			res.ymd.d = tm.tm_mday;
+			res = dt_dconv(DT_YWD, res);
+			break;
+
 		default:
 			break;
 		}
 		break;
 	}
+
 	case DT_DAISY:
 		/* time_t's base is 1970-01-01, which is daisy 19359 */
 		res.daisy = t / 86400 + 19359;
 		break;
+
+		/* the rest doesn't make sense I say */
 	default:
-	case DT_MD:
-		/* doesn't make sense */
 	case DT_DUNK:
 		res.u = 0;
 	}
@@ -1297,6 +1343,9 @@ dt_dconv(dt_dtyp_t tgttyp, struct dt_d_s d)
 {
 	struct dt_d_s res = dt_d_initialiser();
 
+	/* fix up before conversion */
+	d = dt_dfixup(d);
+
 	switch ((res.typ = tgttyp)) {
 	case DT_YMD:
 		res.ymd = dt_conv_to_ymd(d);
@@ -1331,6 +1380,9 @@ dt_dconv(dt_dtyp_t tgttyp, struct dt_d_s d)
 	case DT_YWD:
 		res.ywd = dt_conv_to_ywd(d);
 		break;
+	case DT_YD:
+		res.yd = dt_conv_to_yd(d);
+		break;
 	case DT_DUNK:
 	default:
 		res.typ = DT_DUNK;
@@ -1386,6 +1438,10 @@ dt_dadd_d(struct dt_d_s d, int n)
 		d.ywd = __ywd_add_d(d.ywd, n);
 		break;
 
+	case DT_YD:
+		d.yd = __yd_add_d(d.yd, n);
+		break;
+
 	case DT_DUNK:
 	default:
 		d.typ = DT_DUNK;
@@ -1442,6 +1498,10 @@ dt_dadd_b(struct dt_d_s d, int n)
 		d.ywd = __ywd_add_b(d.ywd, n);
 		break;
 
+	case DT_YD:
+		d.yd = __yd_add_b(d.yd, n);
+		break;
+
 	case DT_DUNK:
 	default:
 		d.typ = DT_DUNK;
@@ -1498,6 +1558,10 @@ dt_dadd_w(struct dt_d_s d, int n)
 		d.ywd = __ywd_add_w(d.ywd, n);
 		break;
 
+	case DT_YD:
+		d.yd = __yd_add_w(d.yd, n);
+		break;
+
 	case DT_DUNK:
 	default:
 		d.typ = DT_DUNK;
@@ -1534,6 +1598,10 @@ dt_dadd_m(struct dt_d_s d, int n)
 		/* ywd have no notion of months */
 		break;
 
+	case DT_YD:
+		/* yd have no notion of months */
+		break;
+
 	case DT_DUNK:
 	default:
 		d.typ = DT_DUNK;
@@ -1570,6 +1638,10 @@ dt_dadd_y(struct dt_d_s d, int n)
 		d.ywd = __ywd_add_y(d.ywd, n);
 		break;
 
+	case DT_YD:
+		d.yd = __yd_add_y(d.yd, n);
+		break;
+
 	case DT_DUNK:
 	default:
 		d.typ = DT_DUNK;
@@ -1580,40 +1652,46 @@ dt_dadd_y(struct dt_d_s d, int n)
 }
 
 DEFUN struct dt_d_s
-dt_dadd(struct dt_d_s d, struct dt_d_s dur)
+dt_dadd(struct dt_d_s d, struct dt_ddur_s dur)
 {
-	struct strpdi_s durcch = strpdi_initialiser();
-
-	__fill_strpdi(&durcch, dur);
-
-	if (durcch.d) {
-		d = dt_dadd_d(d, durcch.d);
-	} else if (durcch.b) {
-		d = dt_dadd_b(d, durcch.b);
-	}
-	if (durcch.w) {
-		d = dt_dadd_w(d, durcch.w);
-	}
-	if (durcch.m) {
-		d = dt_dadd_m(d, durcch.m);
-	}
-	if (durcch.y) {
-		d = dt_dadd_y(d, durcch.y);
+	switch (dur.durtyp) {
+	case DT_DURD:
+		d = dt_dadd_d(d, dur.dv);
+		break;
+	case DT_DURBD:
+		d = dt_dadd_b(d, dur.dv);
+		break;
+	case DT_DURWK:
+		d = dt_dadd_w(d, dur.dv);
+		break;
+	case DT_DURMO:
+		d = dt_dadd_m(d, dur.dv);
+		break;
+	case DT_DURYR:
+		d = dt_dadd_y(d, dur.dv);
+		break;
+	default:
+	case DT_DURUNK:
+		/* huh? */
+		break;
 	}
 	return d;
 }
 
-DEFUN struct dt_d_s
-dt_neg_dur(struct dt_d_s dur)
+DEFUN struct dt_ddur_s
+dt_neg_dur(struct dt_ddur_s dur)
 {
 	dur.neg = (uint16_t)(~dur.neg & 0x01);
-	switch (dur.typ) {
-	case DT_DAISY:
-		dur.daisydur = -dur.daisydur;
-		break;
-	case DT_BIZSI:
-		dur.bizsidur = -dur.bizsidur;
+	switch (dur.durtyp) {
+	case DT_DURD:
+	case DT_DURBD:
+	case DT_DURWK:
+	case DT_DURMO:
+	case DT_DURQU:
+	case DT_DURYR:
+		dur.dv = -dur.dv;
 		break;
+
 	default:
 		break;
 	}
@@ -1621,106 +1699,70 @@ dt_neg_dur(struct dt_d_s dur)
 }
 
 DEFUN int
-dt_dur_neg_p(struct dt_d_s dur)
+dt_dur_neg_p(struct dt_ddur_s dur)
 {
-	switch (dur.typ) {
-	case DT_DAISY:
-		return dur.daisydur < 0;
-	case DT_BIZSI:
-		return dur.bizsidur < 0;
+	switch (dur.durtyp) {
+	case DT_DURD:
+	case DT_DURBD:
+	case DT_DURWK:
+	case DT_DURMO:
+	case DT_DURQU:
+	case DT_DURYR:
+		return dur.dv < 0;
 	default:
-		return dur.neg;
+		break;
 	}
+	return dur.neg;
 }
 
-DEFUN struct dt_d_s
-dt_ddiff(dt_dtyp_t tgttyp, struct dt_d_s d1, struct dt_d_s d2)
+DEFUN struct dt_ddur_s
+dt_ddiff(dt_durtyp_t tgttyp, struct dt_d_s d1, struct dt_d_s d2)
 {
-	struct dt_d_s res = {.typ = DT_DUNK};
-	dt_dtyp_t tmptyp = tgttyp;
+	struct dt_ddur_s res = dt_make_ddur(DT_DURUNK, 0);
 
-	if (tgttyp == DT_MD) {
-		if (d1.typ == DT_YMD || d2.typ == DT_YMD) {
-			tmptyp = DT_YMD;
-		} else if (d1.typ == DT_YMCW || d2.typ == DT_YMCW) {
-			tmptyp = DT_YMCW;
-		} else {
-			tmptyp = DT_DAISY;
-		}
-	}
-
-	switch (tmptyp) {
-	case DT_BIZSI:
-	case DT_DAISY: {
+	switch (tgttyp) {
+	case DT_DURD:
+	case DT_DURBD: {
 		dt_daisy_t tmp1 = dt_conv_to_daisy(d1);
 		dt_daisy_t tmp2 = dt_conv_to_daisy(d2);
 		res = __daisy_diff(tmp1, tmp2);
 
 		/* fix up result in case it's bizsi, i.e. kick weekends */
-		if (tgttyp == DT_BIZSI) {
+		if (tgttyp == DT_DURBD) {
 			dt_dow_t wdb = __daisy_get_wday(tmp2);
-			res.bizsidur = __get_nbdays(res.daisydur, wdb);
+			res.dv = __get_nbdays(res.dv, wdb);
 		}
 		break;
 	}
-	case DT_YMD: {
+	case DT_DURYMD: {
 		dt_ymd_t tmp1 = dt_conv_to_ymd(d1);
 		dt_ymd_t tmp2 = dt_conv_to_ymd(d2);
 		res = __ymd_diff(tmp1, tmp2);
 		break;
 	}
-	case DT_YMCW: {
+	case DT_DURYMCW: {
 		dt_ymcw_t tmp1 = dt_conv_to_ymcw(d1);
 		dt_ymcw_t tmp2 = dt_conv_to_ymcw(d2);
 		res = __ymcw_diff(tmp1, tmp2);
 		break;
 	}
-	case DT_YD: {
+	case DT_DURYD: {
 		dt_yd_t tmp1 = dt_conv_to_yd(d1);
 		dt_yd_t tmp2 = dt_conv_to_yd(d2);
 		res = __yd_diff(tmp1, tmp2);
 		break;
 	}
-	case DT_YWD: {
+	case DT_DURYWD: {
 		dt_ywd_t tmp1 = dt_conv_to_ywd(d1);
 		dt_ywd_t tmp2 = dt_conv_to_ywd(d2);
 		res = __ywd_diff(tmp1, tmp2);
 		break;
 	}
-	case DT_BIZDA:
-	case DT_DUNK:
+	case DT_DURBIZDA:
+	case DT_DURUNK:
 	default:
-		res.typ = DT_DUNK;
-		res.u = 0;
-		/* @fallthrough@ */
-	case DT_MD:
-		/* md is handled later */
 		break;
 	}
-	/* check if we had DT_MD as tgttyp */
-	if (tgttyp == DT_MD) {
-		/* convert res back to DT_MD */
-		struct dt_d_s tmp = {.typ = DT_MD};
-
-		switch (tmptyp) {
-		case DT_YMD:
-			tmp.md.m = res.ymd.y * GREG_MONTHS_P_YEAR + res.ymd.m;
-			tmp.md.d = res.ymd.d;
-			break;
-		case DT_YMCW:
-			tmp.md.m = res.ymcw.y * GREG_MONTHS_P_YEAR + res.ymcw.m;
-			tmp.md.d = res.ymcw.w * GREG_DAYS_P_WEEK + res.ymcw.c;
-			break;
-		case DT_DAISY:
-			tmp.md.m = 0;
-			tmp.md.d = res.daisy;
-			break;
-
-		default:
-			break;
-		}
-		res = tmp;
-	}
 	return res;
 }
 
@@ -1740,6 +1782,7 @@ dt_dcmp(struct dt_d_s d1, struct dt_d_s d2)
 	case DT_DAISY:
 	case DT_BIZDA:
 	case DT_YWD:
+	case DT_YD:
 		/* use arithmetic comparison */
 		if (d1.u == d2.u) {
 			return 0;
@@ -1758,7 +1801,49 @@ dt_dcmp(struct dt_d_s d1, struct dt_d_s d2)
 DEFUN int
 dt_d_in_range_p(struct dt_d_s d, struct dt_d_s d1, struct dt_d_s d2)
 {
-	return dt_dcmp(d, d1) >= 0 && dt_dcmp(d, d2) <= 0;
+/* use the following multiplication table
+ *
+ * |d,d2|v |d,d1|>  -2 -1  0  1
+ *      -2          -1 -1 -1 -1
+ *      -1          -1  0  1  1
+ *       0          -1  0  1  1
+ *       1          -1  0  0  0
+ *
+ * encoded in a 32bit uint */
+	static const uint32_t m = 0b10010111100101111010101111111111U;
+	const unsigned int i = (dt_dcmp(d, d1) + 2) & 0b11U;
+	const unsigned int j = (dt_dcmp(d, d2) + 2) & 0b11U;
+	return 2 - ((m >> (i * 8U + j * 2U)) & 0b11U);
+}
+
+DEFUN __attribute__((pure)) struct dt_d_s
+dt_dfixup(struct dt_d_s d)
+{
+	switch (d.typ) {
+	case DT_YMD:
+		d.ymd = __ymd_fixup(d.ymd);
+		break;
+	case DT_YMCW:
+		d.ymcw = __ymcw_fixup(d.ymcw);
+		break;
+	case DT_YWD:
+		d.ywd = __ywd_fixup(d.ywd);
+		break;
+	case DT_YD:
+		d.yd = __yd_fixup(d.yd);
+		break;
+	case DT_BIZDA:
+		d.bizda = __bizda_fixup(d.bizda);
+		break;
+
+		/* these can't be buggered */
+	case DT_DAISY:
+	case DT_JDN:
+	case DT_LDN:
+	default:
+		break;
+	}
+	return d;
 }
 
 #endif	/* INCLUDED_date_core_c_ */
