@@ -1,6 +1,6 @@
 /*** strops.c -- useful string operations
  *
- * Copyright (C) 2011-2014 Sebastian Freundt
+ * Copyright (C) 2011-2016 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -67,17 +67,38 @@
 DEFUN int32_t
 strtoi_lim(const char *str, const char **ep, int32_t llim, int32_t ulim)
 {
+	const char *sp = str;
 	int32_t res = 0;
-	const char *sp;
-	/* we keep track of the number of digits via rulim */
-	int32_t rulim;
 
-	for (sp = str, rulim = ulim > 10 ? ulim : 10;
-	     res * 10 <= ulim && rulim && *sp >= '0' && *sp <= '9';
-	     sp++, rulim /= 10) {
-		res *= 10;
-		res += *sp - '0';
+	/* we keep track of the number of digits via rulim */
+	for (int32_t rulim = ulim > 10 ? ulim : 10;
+	     rulim && (unsigned char)(*sp ^ '0') < 10U &&
+		     (res *= 10, res += (unsigned char)(*sp++ ^ '0')) <= ulim;
+	     rulim /= 10);
+	if (UNLIKELY(sp == str)) {
+		res = -1;
+	} else if (UNLIKELY(res < llim || res > ulim)) {
+		res = -2;
 	}
+	*ep = (char*)sp;
+	return res;
+}
+
+DEFUN int32_t
+padstrtoi_lim(const char *str, const char **ep, int32_t llim, int32_t ulim)
+{
+	const char *sp;
+	int32_t rulim = ulim > 10 ? ulim : 10;
+	int32_t res = 0;
+
+	/* overread whitespace */
+	for (; *str == ' '; str++, rulim /= 10);
+
+	/* we keep track of the number of digits via rulim */
+	for (sp = str;
+	     rulim && (unsigned char)(*sp ^ '0') < 10U &&
+		     (res *= 10, res += (unsigned char)(*sp++ ^ '0')) <= ulim;
+	     rulim /= 10);
 	if (UNLIKELY(sp == str)) {
 		res = -1;
 	} else if (UNLIKELY(res < llim || res > ulim)) {
@@ -92,17 +113,21 @@ strtoi(const char *str, const char **ep)
 {
 	const char *sp = str;
 	bool negp = false;
-	int32_t res;
+	int32_t res = 0;
 
 	if (*str == '-') {
 		negp = true;
 		sp++;
 	}
-	if ((res = strtoi_lim(sp, ep, 0, INT32_MAX)) < 0) {
-		*ep = str;
+	while (res < INT32_MAX / 10 && (unsigned char)(*sp ^ '0') < 10U) {
+		res *= 10, res += (unsigned char)(*sp++ ^ '0');
+	}
+	if (UNLIKELY(sp == str)) {
+		res = -1;
 	} else if (negp) {
 		res = -res;
 	}
+	*ep = (char*)sp;
 	return res;
 }
 
@@ -110,7 +135,7 @@ DEFUN size_t
 ui32tostr(char *restrict buf, size_t bsz, uint32_t d, int width)
 {
 /* all strings should be little */
-#define C(x, d)	(char)((x) / (d) % 10 + '0')
+#define C(x, d)	(char)(((x) / (d) % 10) ^ '0')
 	size_t res;
 
 	if (UNLIKELY(d > 1000000000U)) {
@@ -151,7 +176,7 @@ DEFUN size_t
 ui32topstr(char *restrict b, size_t z, uint32_t d, int width, char pad)
 {
 /* all strings should be little */
-#define C(x)	(char)((x) + '0')
+#define C(x)	(char)((x) ^ '0')
 	char *restrict bp = b;
 	const char *const ep = b + z;
 
@@ -401,8 +426,8 @@ DEFUN int32_t
 strtoarri(const char *buf, const char **ep, const char *const *arr, size_t narr)
 {
 /* take a string, compare it to an array of string (case-insensitively) and
- * return its index if found or -1 if not */
-	for (size_t i = 0; i < narr; i++) {
+ * return its index if found or 0 if not */
+	for (size_t i = 1U; i < narr; i++) {
 		const char *chk = arr[i];
 		size_t len = strlen(chk);
 
@@ -519,6 +544,16 @@ xstrpbrkp(const char *src, const char *set, size_t *set_offs)
 	return (char*)p;
 }
 
+DEFUN char*
+xmempbrk(const char *src, size_t len, const char *set)
+{
+	size_t i;
+
+	set_up_table((const unsigned char*)set, false);
+	for (i = 0U; i < len && !in_current_set((unsigned char)src[i]); i++);
+	return (char*)src + i;
+}
+
 #if defined __INTEL_COMPILER
 # pragma warning (default:2203)
 #elif defined __GNUC__
