@@ -1,6 +1,6 @@
 /*** ywd.c -- guts for ywd dates
  *
- * Copyright (C) 2012-2014 Sebastian Freundt
+ * Copyright (C) 2012-2016 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -70,7 +70,8 @@ __ywd_get_jan01_wday(dt_ywd_t d)
 /* hang of 0 means Mon, -1 Tue, -2 Wed, -3 Thu, 3 Fri, 2 Sat, 1 Sun */
 	int res;
 
-	assert(d.hang >= -3 && d.hang <= 3);
+	/* the second portion of the assert is trivially true */
+	assert(d.hang >= -3 /*&& d.hang <= 3*/);
 	if (UNLIKELY((res = 1 - d.hang) <= 0)) {
 		res += GREG_DAYS_P_WEEK;
 		assert(res > DT_MIRACLEDAY);
@@ -107,7 +108,7 @@ __ywd_get_dec31_wday(dt_ywd_t d)
 }
 
 #if defined GET_ISOWK_FULL_SWITCH
-static __attribute__((const, pure)) unsigned int
+DEFUN __attribute__((const, pure)) inline unsigned int
 __get_isowk(unsigned int y)
 {
 /* return the number of iso weeks in Y */
@@ -364,7 +365,7 @@ __get_z31wk(unsigned int y)
 }
 
 #elif defined GET_ISOWK_28Y_SWITCH
-static inline __attribute__((const, pure)) unsigned int
+DEFUN __attribute__((const, pure)) inline unsigned int
 __get_isowk(unsigned int y)
 {
 	switch (y % 28U) {
@@ -415,7 +416,7 @@ __get_z31wk(unsigned int y)
 
 #endif	/* GET_ISOWK_* */
 
-static __attribute__((pure)) dt_ywd_t
+DEFUN __attribute__((pure)) dt_ywd_t
 __ywd_fixup(dt_ywd_t d)
 {
 /* given dates like 2012-W53-01 this returns 2013-W01-01 */
@@ -520,11 +521,7 @@ __make_ywd_c(unsigned int y, unsigned int c, dt_dow_t w, unsigned int cc)
 	res.c = c;
 	res.w = w;
 	res.hang = hang;
-#if defined WITH_FAST_ARITH
 	return res;
-#else  /* !WITH_FAST_ARITH */
-	return __ywd_fixup(res);
-#endif	/* WITH_FAST_ARITH */
 }
 
 static dt_ywd_t
@@ -592,11 +589,13 @@ __make_ywd_ybd(unsigned int y, int yd)
 	return res;
 }
 
-static unsigned int
+static __attribute__((pure, const)) int
 __ywd_get_yday(dt_ywd_t d)
 {
 /* since everything is in ISO 8601 format, getting the doy is a matter of
- * counting how many days there are in a week. */
+ * counting how many days there are in a week.
+ * This may return negative values and values larger than the number of
+ * days in that year. */
 	return GREG_DAYS_P_WEEK * (d.c - 1) + d.w + d.hang;
 }
 
@@ -611,7 +610,7 @@ __ywd_get_wcnt_mon(dt_ywd_t d)
 {
 /* given a YWD with week-count within the year (ISOWK_CNT convention)
  * return the week-count within the month (ABSWK_CNT convention) */
-	unsigned int yd = __ywd_get_yday(d);
+	int yd = __ywd_get_yday(d);
 	struct __md_s x = __yday_get_md(d.y, yd);
 	return (x.d - 1) / 7 + 1;
 }
@@ -653,7 +652,7 @@ __ywd_get_wcnt_year(dt_ywd_t d, unsigned int tgtcc)
 static struct __md_s
 __ywd_get_md(dt_ywd_t d)
 {
-	unsigned int yday = __ywd_get_yday(d);
+	int yday = __ywd_get_yday(d);
 	struct __md_s res = __yday_get_md(d.y, yday);
 
 	if (UNLIKELY(res.m == 0)) {
@@ -668,7 +667,7 @@ __ywd_get_md(dt_ywd_t d)
 static unsigned int
 __ywd_get_mon(dt_ywd_t d)
 {
-	unsigned int yd = __ywd_get_yday(d);
+	int yd = __ywd_get_yday(d);
 	return __yday_get_md(d.y, yd).m;
 }
 
@@ -762,14 +761,24 @@ __ywd_to_daisy(dt_ywd_t d)
 static dt_yd_t
 __ywd_to_yd(dt_ywd_t d)
 {
+	unsigned int y = __ywd_get_year(d);
+	int x = __ywd_get_yday(d);
+
+	if (UNLIKELY(x <= 0)) {
+		/* gotta go for last years thing */
+		x += __get_ydays(d.y - 1);
+	} else if (x > (int)__get_ydays(d.y)) {
+		x -= __get_ydays(d.y);
+	}
+
 #if defined HAVE_ANON_STRUCTS_INIT
-	return (dt_yd_t){.y = d.y, .d = __ywd_get_yday(d)};
-#else
+	return (dt_yd_t){.y = y, .d = x};
+#else  /* !HAVE_ANON_STRUCTS_INIT */
 	dt_yd_t res;
-	res.y = d.y;
-	res.d = __ywd_get_yday(d);
+	res.y = y;
+	res.d = x;
 	return res;
-#endif
+#endif	/* HAVE_ANON_STRUCTS_INIT */
 }
 #endif	/* ASPECT_CONV */
 
@@ -865,7 +874,7 @@ __ywd_add_y(dt_ywd_t d, int n)
 	/* recompute hang */
 	j01 = __get_jan01_wday(d.y);
 	d.hang = __ywd_get_jan01_hang(j01);
-	return __ywd_fixup(d);
+	return d;
 }
 #endif	/* ASPECT_ADD */
 
@@ -873,11 +882,11 @@ __ywd_add_y(dt_ywd_t d, int n)
 #if defined ASPECT_DIFF && !defined YWD_ASPECT_DIFF_
 #define YWD_ASPECT_DIFF_
 
-static struct dt_d_s
+static struct dt_ddur_s
 __ywd_diff(dt_ywd_t d1, dt_ywd_t d2)
 {
 /* compute d2 - d1 entirely in terms of ymd but express the result as yd */
-	struct dt_d_s res = {.typ = DT_YWD, .dur = 1};
+	struct dt_ddur_s res = dt_make_ddur(DT_DURYWD, 0);
 	signed int tgtd;
 	signed int tgtw;
 	signed int tgty;
@@ -922,12 +931,13 @@ DEFUN void
 __prep_strfd_ywd(struct strpd_s *tgt, dt_ywd_t d)
 {
 /* place ywd data of THIS into D for printing with FMT. */
-	if (d.c == 1 && d.w < __ywd_get_jan01_wday(d) && d.w) {
+	if (d.c == 1 && d.hang < 0 && d.w < __ywd_get_jan01_wday(d)) {
 		/* put gregorian year into y and real year into q */
 		tgt->y = d.y - 1;
 		tgt->q = d.y;
-	} else if (d.c >= 53) {
-		/* bit of service for the %Y printer */
+	} else if (d.c >= 53 && d.w >= __ywd_get_jan01_wday(d) +
+		   1U/*coz 365 = 1 mod 7*/ + __leapp(d.y)) {
+		/* put gregorian year into y and real year into q */
 		tgt->y = d.y + 1;
 		tgt->q = d.y;
 	} else {
