@@ -1,6 +1,6 @@
 /*** ddiff.c -- perform simple date arithmetic, date minus date
  *
- * Copyright (C) 2011-2014 Sebastian Freundt
+ * Copyright (C) 2011-2016 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -46,6 +46,7 @@
 #include "date-core-private.h"
 #include "dt-core-private.h"
 #include "dt-io.h"
+#include "dt-locale.h"
 #include "prchunk.h"
 
 #if !defined UNUSED
@@ -104,6 +105,9 @@ determine_durfmt(const char *fmt)
 			res.has_day = 1;
 			res.has_biz = 1;
 			break;
+		case DT_DAISY:
+			res.has_day = 1;
+			break;
 		case DT_BIZSI:
 			res.has_day = 1;
 			res.has_biz = 1;
@@ -178,7 +182,7 @@ determine_durfmt(const char *fmt)
 	return res;
 }
 
-static dt_dttyp_t
+static dt_dtdurtyp_t
 determine_durtype(struct dt_dt_s d1, struct dt_dt_s d2, durfmt_t f)
 {
 	/* the type-multiplication table looks like:
@@ -195,27 +199,28 @@ determine_durtype(struct dt_dt_s d1, struct dt_dt_s d2, durfmt_t f)
 		;
 	} else if (dt_sandwich_only_t_p(d1) || dt_sandwich_only_t_p(d2)) {
 		/* isn't defined */
-		return (dt_dttyp_t)DT_UNK;
+		return (dt_dtdurtyp_t)DT_DURUNK;
 	} else if (f.has_week && f.has_mon) {
-		return (dt_dttyp_t)DT_YMCW;
+		return (dt_dtdurtyp_t)DT_DURYMCW;
 	} else if (f.has_week && f.has_year) {
-		return (dt_dttyp_t)DT_YWD;
+		return (dt_dtdurtyp_t)DT_DURYWD;
 	} else if (f.has_mon) {
-		return (dt_dttyp_t)DT_YMD;
+		return (dt_dtdurtyp_t)DT_DURYMD;
 	} else if (f.has_year && f.has_day) {
-		return (dt_dttyp_t)DT_YD;
+		return (dt_dtdurtyp_t)DT_DURYD;
 	} else if (f.has_day && f.has_biz) {
-		return (dt_dttyp_t)DT_BIZSI;
+		return (dt_dtdurtyp_t)DT_DURBD;
 	} else if (f.has_year) {
-		return (dt_dttyp_t)DT_MD;
+		return (dt_dtdurtyp_t)DT_DURYMD;
 	} else if (dt_sandwich_only_d_p(d1) || dt_sandwich_only_d_p(d2)) {
 		/* default date-only type */
-		return (dt_dttyp_t)DT_DAISY;
+		return (dt_dtdurtyp_t)DT_DURD;
 	} else if (UNLIKELY(f.has_tai)) {
 		/* we has tais */
-		return (dt_dttyp_t)DT_SEXYTAI;
+		return (dt_dtdurtyp_t)0xffU;
 	}
-	return (dt_dttyp_t)DT_SEXY;
+	/* otherwise */
+	return DT_DURS;
 }
 
 
@@ -273,108 +278,115 @@ duration between `%s' and `%s' is not defined", d1, d2);
 	return;
 }
 
-static long int
-__strf_tot_secs(struct dt_dt_s dur)
+static __attribute__((pure)) long int
+__strf_tot_secs(struct dt_dtdur_s dur)
 {
-	long int res;
+/* return time portion of duration in UTC seconds */
+	long int s = dur.dv;
 
-	if (dur.typ == DT_SEXY) {
-		res = dur.sexydur;
-	} else if (dur.typ == DT_SEXYTAI) {
-		res = dur.soft;
-	} else {
-		/* otherwise */
-		res = dur.t.sdur;
+	if (UNLIKELY(dur.tai) && dur.durtyp == DT_DURS) {
+		return dur.soft - dur.corr;
 	}
-	return res;
+
+	switch (dur.durtyp) {
+	default:
+		/* all the date types */
+		return dur.t.sdur;
+	case DT_DURH:
+		s *= MINS_PER_HOUR;
+		/*@fallthrough@*/
+	case DT_DURM:
+		s *= SECS_PER_MIN;
+		/*@fallthrough@*/
+	case DT_DURS:
+		break;
+	case DT_DURNANO:
+		s /= NANOS_PER_SEC;
+		break;
+	}
+	return s;
 }
 
-static long int
-__strf_tot_corr(struct dt_dt_s dur)
+static __attribute__((pure)) long int
+__strf_tot_corr(struct dt_dtdur_s dur)
 {
-	if (dur.typ == DT_SEXYTAI) {
+	if (dur.durtyp == DT_DURS && dur.tai) {
 		return dur.corr;
 	}
 	/* otherwise no corrections */
 	return 0;
 }
 
-static int
-__strf_tot_days(struct dt_dt_s dur)
+static __attribute__((pure)) int
+__strf_tot_days(struct dt_dtdur_s dur)
 {
-/* DUR expressed solely as the number of days */
-	int d = 0;
+/* return date portion of DURation in days */
+	int d;
 
-	switch (dur.d.typ) {
-	case DT_DAISY:
-		d = dur.d.daisydur;
+	switch (dur.d.durtyp) {
+	case DT_DURD:
+	case DT_DURBD:
+		d = dur.d.dv;
 		break;
-	case DT_BIZSI:
-		d = dur.d.bizsidur;
-		break;
-	case DT_BIZDA:
+	case DT_DURBIZDA:
 		d = dur.d.bizda.bd;
 		break;
-	case DT_YMD:
+	case DT_DURYMD:
 		d = dur.d.ymd.d;
 		break;
-	case DT_YD:
+	case DT_DURYD:
 		d = dur.d.yd.d;
 		break;
-	case DT_YMCW:
-		d = dur.d.ymcw.w + dur.d.ymcw.c * (int)GREG_DAYS_P_WEEK;;
+	case DT_DURYMCW:
+		d = dur.d.ymcw.w + dur.d.ymcw.c * (int)GREG_DAYS_P_WEEK;
 		break;
-	case DT_YWD:
+	case DT_DURYWD:
 		d = dur.d.ywd.w + dur.d.ywd.c * (int)GREG_DAYS_P_WEEK;
 		break;
-	case DT_MD:
-		d = dur.d.md.d;
-		break;
 	default:
+		d = 0;
 		break;
 	}
 	return d;
 }
 
-static int
-__strf_tot_mon(struct dt_dt_s dur)
+static __attribute__((pure)) int
+__strf_tot_mon(struct dt_dtdur_s dur)
 {
 /* DUR expressed as month and days */
-	int m = 0;
+	int m;
 
-	switch (dur.d.typ) {
-	case DT_BIZDA:
+	switch (dur.d.durtyp) {
+	case DT_DURBIZDA:
 		m = dur.d.bizda.m + dur.d.bizda.y * (int)GREG_MONTHS_P_YEAR;
 		break;
-	case DT_YMD:
+	case DT_DURYMD:
 		m = dur.d.ymd.m + dur.d.ymd.y * (int)GREG_MONTHS_P_YEAR;
 		break;
-	case DT_MD:
-		m = dur.d.md.m;
-		break;
-	case DT_YMCW:
+	case DT_DURYMCW:
 		m = dur.d.ymcw.m + dur.d.ymcw.y * (int)GREG_MONTHS_P_YEAR;
 		break;
-	case DT_YD:
+	case DT_DURYD:
 		m = dur.d.yd.y * (int)GREG_MONTHS_P_YEAR;
 		break;
-	case DT_YWD:
+	case DT_DURYWD:
 		m = dur.d.ywd.y * (int)GREG_MONTHS_P_YEAR;
 		break;
 	default:
+		m = 0;
 		break;
 	}
 	return m;
 }
 
-static int
-__strf_ym_mon(struct dt_dt_s dur)
+static __attribute__((pure)) int
+__strf_ym_mon(struct dt_dtdur_s dur)
 {
 	return __strf_tot_mon(dur) % (int)GREG_MONTHS_P_YEAR;
 }
 
-static int
-__strf_tot_years(struct dt_dt_s dur)
+static __attribute__((pure)) int
+__strf_tot_years(struct dt_dtdur_s dur)
 {
 	return __strf_tot_mon(dur) / (int)GREG_MONTHS_P_YEAR;
 }
@@ -392,13 +404,14 @@ static struct precalc_s {
 	long int N;
 
 	long int rS;
-} precalc(durfmt_t f, struct dt_dt_s dur)
+} precalc(durfmt_t f, struct dt_dtdur_s dur)
 {
 #define MINS_PER_DAY	(MINS_PER_HOUR * HOURS_PER_DAY)
 #define SECS_PER_WEEK	(SECS_PER_DAY * GREG_DAYS_P_WEEK)
 #define MINS_PER_WEEK	(MINS_PER_DAY * GREG_DAYS_P_WEEK)
 #define HOURS_PER_WEEK	(HOURS_PER_DAY * GREG_DAYS_P_WEEK)
 	struct precalc_s res = {0};
+	long long int us;
 
 	/* date specs */
 	if (f.has_year) {
@@ -412,63 +425,57 @@ static struct precalc_s {
 		/* just months */
 		res.m = __strf_tot_mon(dur);
 	}
-	if (f.has_day || f.has_week) {
-		res.d = __strf_tot_days(dur);
-		if (f.has_week) {
-			/* week shadows days in the hierarchy */
-			res.w = res.d / (int)GREG_DAYS_P_WEEK;
-			res.d %= (int)GREG_DAYS_P_WEEK;
-		}
+
+	/* the other units are easily converted as their factors are fixed.
+	 * we operate on clean seconds and attribute leap seconds only
+	 * to the S slot, so 59 seconds plus a leap second != 1 minute */
+	with (long long int S = __strf_tot_secs(dur), d = __strf_tot_days(dur)) {
+		us = d * (int)SECS_PER_DAY + S;
 	}
 
-	/* time specs */
-	if (f.has_sec || f.has_min || f.has_hour) {
-		res.S = __strf_tot_secs(dur);
+	if (f.has_week) {
+		/* week shadows days in the hierarchy */
+		res.w = us / (int)SECS_PER_WEEK;
+		us %= (int)SECS_PER_WEEK;
+	}
+	if (f.has_day) {
+		res.d += us / (int)SECS_PER_DAY;
+		us %= (int)SECS_PER_DAY;
+	}
+	if (f.has_hour) {
+		res.H = us / (long int)SECS_PER_HOUR;
+		us %= (long int)SECS_PER_HOUR;
+	}
+	if (f.has_min) {
+		/* minutes and seconds */
+		res.M = us / (long int)SECS_PER_MIN;
+		us %= (long int)SECS_PER_MIN;
+	}
+	if (f.has_sec) {
+		res.S = us + __strf_tot_corr(dur);
+	}
 
-		/* carry from the date specs */
-		res.S += res.w * (long int)SECS_PER_WEEK;
-		res.S += res.d * (long int)SECS_PER_DAY;
-
-		if (f.has_week) {
-			res.w = res.S / (long int)SECS_PER_WEEK;
-			res.S %= (long int)SECS_PER_WEEK;
-		}
-		if (f.has_day) {
-			res.d = res.S / (long int)SECS_PER_DAY;
-			res.S %= (long int)SECS_PER_DAY;
-		}
-		if (f.has_hour) {
-			res.H = res.S / (long int)SECS_PER_HOUR;
-			res.S %= (long int)SECS_PER_HOUR;
-		}
-		if (f.has_min) {
-			/* minutes and seconds */
-			res.M = res.S / (long int)SECS_PER_MIN;
-			res.S %= (long int)SECS_PER_MIN;
-		}
-
-		/* just in case the duration iss negative jump through all
-		 * the hoops again, backwards */
-		if (res.w < 0 || res.d < 0 ||
-		    res.H < 0 || res.M < 0 || res.S < 0) {
-			if (0) {
-			fixup_d:
-				res.d = -res.d;
-			fixup_H:
-				res.H = -res.H;
-			fixup_M:
-				res.M = -res.M;
-			fixup_S:
-				res.S = -res.S;
-			} else if (f.has_week) {
-				goto fixup_d;
-			} else if (f.has_day) {
-				goto fixup_H;
-			} else if (f.has_hour) {
-				goto fixup_M;
-			} else if (f.has_min) {
-				goto fixup_S;
-			}
+	/* just in case the duration iss negative jump through all
+	 * the hoops again, backwards */
+	if (res.w < 0 || res.d < 0 ||
+	    res.H < 0 || res.M < 0 || res.S < 0) {
+		if (0) {
+		fixup_d:
+			res.d = -res.d;
+		fixup_H:
+			res.H = -res.H;
+		fixup_M:
+			res.M = -res.M;
+		fixup_S:
+			res.S = -res.S;
+		} else if (f.has_week) {
+			goto fixup_d;
+		} else if (f.has_day) {
+			goto fixup_H;
+		} else if (f.has_hour) {
+			goto fixup_M;
+		} else if (f.has_min) {
+			goto fixup_S;
 		}
 	}
 	return res;
@@ -477,7 +484,7 @@ static struct precalc_s {
 static size_t
 __strfdtdur(
 	char *restrict buf, size_t bsz, const char *fmt,
-	struct dt_dt_s dur, durfmt_t f)
+	struct dt_dtdur_s dur, durfmt_t f, bool only_d_p)
 {
 /* like strfdtdur() but do some calculations based on F on the way there */
 	static const char sexy_dflt_dur[] = "%0T";
@@ -492,39 +499,16 @@ __strfdtdur(
 	}
 
 	/* translate high-level format names */
-	if (dur.typ == DT_SEXY) {
-		if (fmt == NULL) {
-			fmt = sexy_dflt_dur;
-			f.has_sec = 1U;
-		}
-	} else if (dur.typ == DT_SEXYTAI) {
-		if (fmt == NULL) {
-			fmt = sexy_dflt_dur;
-			f.has_sec = 1U;
-		}
-	} else if (dt_sandwich_p(dur)) {
-		if (fmt == NULL) {
-			fmt = sexy_dflt_dur;
-			f.has_sec = 1U;
-		} else {
-			__trans_dtdurfmt(&fmt);
-		}
-	} else if (dt_sandwich_only_d_p(dur)) {
-		if (fmt == NULL) {
-			fmt = ddur_dflt_dur;
-			f.has_day = 1U;
-		} else {
-			__trans_ddurfmt(&fmt);
-			f.has_day = 1U;
-		}
-	} else if (dt_sandwich_only_t_p(dur)) {
-		if (fmt == NULL) {
-			fmt = sexy_dflt_dur;
-			f.has_sec = 1U;
-		}
+	if (fmt == NULL && dur.durtyp >= (dt_dtdurtyp_t)DT_NDURTYP) {
+		fmt = sexy_dflt_dur;
+		f.has_sec = 1U;
+	} else if (fmt == NULL) {
+		fmt = ddur_dflt_dur;
+		f.has_day = 1U;
+	} else if (only_d_p) {
+		__trans_ddurfmt(&fmt);
 	} else {
-		bp = buf;
-		goto out;
+		__trans_dtdurfmt(&fmt);
 	}
 
 	/* precompute */
@@ -577,7 +561,10 @@ __strfdtdur(
 		bizda_suffix:
 			if (spec.bizda) {
 				/* don't print the b after an ordinal */
-				switch (__get_bizda_param(dur.d).ab) {
+				dt_bizda_param_t bprm;
+
+				bprm.bs = dur.d.param;
+				switch (bprm.ab) {
 				case BIZDA_AFTER:
 					*bp++ = 'b';
 					break;
@@ -638,11 +625,11 @@ out:
 }
 
 static int
-ddiff_prnt(struct dt_dt_s dur, const char *fmt, durfmt_t f)
+ddiff_prnt(struct dt_dtdur_s dur, const char *fmt, durfmt_t f, bool only_d_p)
 {
 /* this is mainly a better dt_strfdtdur() */
 	char buf[256];
-	size_t res = __strfdtdur(buf, sizeof(buf), fmt, dur, f);
+	size_t res = __strfdtdur(buf, sizeof(buf), fmt, dur, f, only_d_p);
 
 	if (res > 0 && buf[res - 1] != '\n') {
 		/* auto-newline */
@@ -668,7 +655,7 @@ main(int argc, char *argv[])
 	size_t nfmt;
 	int rc = 0;
 	durfmt_t dfmt;
-	dt_dttyp_t dtyp;
+	dt_dtdurtyp_t dtyp;
 	zif_t fromz = NULL;
 
 	if (yuck_parse(argi, argc, argv)) {
@@ -680,10 +667,18 @@ main(int argc, char *argv[])
 		dt_io_unescape(argi->format_arg);
 	}
 
+	if (argi->from_locale_arg) {
+		setilocale(argi->from_locale_arg);
+	}
+
 	/* try and read the from and to time zones */
 	if (argi->from_zone_arg) {
 		fromz = dt_io_zone(argi->from_zone_arg);
 	}
+	if (argi->base_arg) {
+		struct dt_dt_s base = dt_strpdt(argi->base_arg, NULL, NULL);
+		dt_set_base(base);
+	}
 
 	ofmt = argi->format_arg;
 	fmt = argi->input_format_args;
@@ -707,8 +702,9 @@ main(int argc, char *argv[])
 	if (argi->nargs > 1) {
 		for (size_t i = 1; i < argi->nargs; i++) {
 			struct dt_dt_s d2;
-			struct dt_dt_s dur;
+			struct dt_dtdur_s dur;
 			const char *inp = argi->args[i];
+			bool onlydp;
 
 			d2 = dt_io_strpdt(inp, fmt, nfmt, fromz);
 			if (dt_unk_p(d2)) {
@@ -721,7 +717,9 @@ main(int argc, char *argv[])
 				rc = 2;
 			}
 			/* guess the diff type */
-			if ((dtyp = determine_durtype(d, d2, dfmt)) == DT_UNK) {
+			onlydp = dt_sandwich_only_d_p(d) ||
+				dt_sandwich_only_d_p(d2);
+			if (!(dtyp = determine_durtype(d, d2, dfmt))) {
 				if (!argi->quiet_flag) {
 				        dt_io_warn_dur(refinp, inp);
 					rc = 2;
@@ -730,7 +728,7 @@ main(int argc, char *argv[])
 			}
 			/* subtraction and print */
 			dur = dt_dtdiff(dtyp, d, d2);
-			ddiff_prnt(dur, ofmt, dfmt);
+			ddiff_prnt(dur, ofmt, dfmt, onlydp);
 		}
 	} else {
 		/* read from stdin */
@@ -748,7 +746,8 @@ main(int argc, char *argv[])
 		while (prchunk_fill(pctx) >= 0) {
 			for (char *line; prchunk_haslinep(pctx); lno++) {
 				struct dt_dt_s d2;
-				struct dt_dt_s dur;
+				struct dt_dtdur_s dur;
+				bool onlydp;
 
 				(void)prchunk_getline(pctx, &line);
 				d2 = dt_io_strpdt(line, fmt, nfmt, fromz);
@@ -758,14 +757,19 @@ main(int argc, char *argv[])
 						dt_io_warn_strpdt(line);
 						rc = 2;
 					}
+					if (argi->skip_illegal_flag) {
+						/* empty line */
+						__io_write("\n", 1U, stdout);
+					}
 					continue;
 				} else if (UNLIKELY(d2.fix) &&
 					   !argi->quiet_flag) {
 					rc = 2;
 				}
 				/* guess the diff type */
-				dtyp = determine_durtype(d, d2, dfmt);
-				if (dtyp == DT_UNK) {
+				onlydp = dt_sandwich_only_d_p(d) ||
+					dt_sandwich_only_d_p(d2);
+				if (!(dtyp = determine_durtype(d, d2, dfmt))) {
 					if (!argi->quiet_flag) {
 						dt_io_warn_dur(refinp, line);
 						rc = 2;
@@ -774,7 +778,7 @@ main(int argc, char *argv[])
 				}
 				/* perform subtraction now */
 				dur = dt_dtdiff(dtyp, d, d2);
-				ddiff_prnt(dur, ofmt, dfmt);
+				ddiff_prnt(dur, ofmt, dfmt, onlydp);
 			}
 		}
 		/* get rid of resources */
@@ -782,6 +786,9 @@ main(int argc, char *argv[])
 	}
 
 	dt_io_clear_zones();
+	if (argi->from_locale_arg) {
+		setilocale(NULL);
+	}
 
 out:
 	yuck_free(argi);
