@@ -11,8 +11,8 @@ main(void)
 	struct dt_dt_s dt;
 
 	memset(&dt, 0, sizeof(dt));
-	dt.sexy = 0x1fffffffffffff;
-	if (dt.typ != DT_UNK || dt.dur || dt.neg) {
+	dt.sexy = 0x1fffffffffffffLL;
+	if (dt.typ != DT_UNK || dt.xxx || dt.neg) {
 		return 1;
 	}
 	return 0;
